/**
 * 下载 app serve client的包的命令
 */

import { chalk } from '@/module/chalk.ts';
import { program, Command } from '../../../program.ts';
import fs from 'fs';
import { Readable } from 'stream';
import * as tar from 'tar';
import path from 'path';

// Utility function to convert a web ReadableStream to a Node.js Readable stream
function nodeReadableStreamFromWeb(webStream: ReadableStream<Uint8Array>) {
  const reader = webStream.getReader();
  return new Readable({
    async read() {
      const { done, value } = await reader.read();
      if (done) {
        this.push(null);
      } else {
        this.push(Buffer.from(value));
      }
    },
  });
}

export const appCommand = new Command('micro-app').description('micro-app 命令').action(() => {
  console.log('micro-app');
});

program.addCommand(appCommand);

// https://kevisual.xiongxiao.me/api/micro-app/download/file?notNeedToken=y&title=mark-0.0.2.tgz
const downloadAppCommand = new Command('download')
  .description('下载 app serve client的包')
  .option('-i, --id <id>', '下载 app serve client的包, id 或者title, mark-0.0.2.tgz')
  .option('-o, --output <output>', '下载 app serve client的包, 输出路径')
  .option('-x, --extract <extract>', '下载 app serve client的包, 解压, 默认解压到当前目录')
  .action(async (options) => {
    const id = options.id || '';
    if (!id) {
      console.error(chalk.red('id is required'));
      return;
    }
    let title = '';
    if (id.includes('.tgz')) {
      title = id;
    }

    let curlUrl = `https://kevisual.xiongxiao.me/api/micro-app/download/${id}?notNeedToken=y`;
    if (title) {
      curlUrl = `https://kevisual.xiongxiao.me/api/micro-app/download/file?notNeedToken=y&title=${title}`;
    }
    console.log('download url', curlUrl);
    fetch(curlUrl)
      .then((res) => {
        const contentDisposition = res.headers.get('content-disposition');
        let filename = 'downloaded-file.tgz'; // Default filename

        if (contentDisposition) {
          const match = contentDisposition.match(/filename="?(.+)"?/);
          if (match && match[1]) {
            filename = match[1].replace(/^"|"$/g, '');
          }
        }
        console.log('filename', filename, contentDisposition);
        const outputPath = options.output || filename;
        const fileStream = fs.createWriteStream(outputPath);

        if (res.body) {
          nodeReadableStreamFromWeb(res.body).pipe(fileStream);

          fileStream.on('finish', async () => {
            console.log(chalk.green(`下载成功: ${outputPath}`));
            if (options.extract) {
              console.log(chalk.green(`解压: ${outputPath}`));
              const extractPath = path.join(process.cwd(), options.extract || '.');
              console.log('extractPath', extractPath);
              // const res = await tar.extract({
              //   file: outputPath,
              //   cwd: extractPath,
              // });
              // console.log('res', res);
              // untar.extract({ file: outputPath, cwd: extractPath });
            }
          });

          fileStream.on('error', (err) => {
            console.error(chalk.red('文件写入错误:', err));
            fileStream.close(); // Ensure the stream is closed on error
          });
        } else {
          console.error(chalk.red('下载失败: 无法获取文件流'));
        }
      })
      .catch((err) => {
        console.error(chalk.red('下载请求失败:', err));
      });
  });

appCommand.addCommand(downloadAppCommand);
